@@ -1,6 +1,6 @@
 #!/usr/bin/python3
 
-from subprocess import Popen,PIPE,call,STDOUT
+from subprocess import Popen, PIPE, call, STDOUT
 import os
 import signal
 import re
@@ -19,13 +19,14 @@ gi.require_version('XApp', '1.0')
 from gi.repository import GObject, Gio, Polkit, Gtk, GLib, UDisks, XApp
 
 try:
+    gi.require_version('Unity', '7.0')
     from gi.repository import Unity
     Using_Unity = True
-except ImportError:
+except Exception:
     Using_Unity = False
 
 if Using_Unity:
-    launcher = Unity.LauncherEntry.get_for_desktop_id ("mintstick.desktop")
+    launcher = Unity.LauncherEntry.get_for_desktop_id("mintstick.desktop")
 
 APP = 'mintstick'
 LOCALE_DIR = "/usr/share/linuxmint/locale"
@@ -37,21 +38,28 @@ _ = gettext.gettext
 # https://technet.microsoft.com/en-us/library/bb490925.aspx
 FORBIDDEN_CHARS = ["*", "?", "/", "\\", "|", ".", ",", ";", ":", "+", "=", "[", "]", "<", ">", "\""]
 
-GObject.threads_init()
+
+# https://wiki.gnome.org/PyGObject/Threading
+# GObject.threads_init()
+
 
 def print_timing(func):
     def wrapper(*arg):
         t1 = time.time()
         res = func(*arg)
         t2 = time.time()
-        print('%s took %0.3f ms' % (func.__name__, (t2-t1)*1000.0))
+        print('%s took %0.3f ms' % (func.__name__, (t2 - t1) * 1000.0))
         return res
+
     return wrapper
 
-class MintStick:
-    def __init__(self, iso_path=None, usb_path=None, filesystem=None, mode=None, debug=False):
 
-        self.debug = debug
+# noinspection PyUnusedLocal
+class MintStick:
+    def __init__(self, iso_path_arg=None, usb_path_arg=None, filesystem_arg=None, mode_arg=None, debug_arg=False):
+
+        self.debug = debug_arg
+        self.filesystem = filesystem_arg
 
         def devices_changed_callback(client):
             self.get_devices()
@@ -65,6 +73,8 @@ class MintStick:
 
         self.process = None
         self.source_id = None
+        self.dev = None
+        self.write_progress = None
 
         self.wTree.set_translation_domain(APP)
 
@@ -73,10 +83,10 @@ class MintStick:
         self.ddpid = 0
 
         self.emergency_dialog = self.wTree.get_object("emergency_dialog")
-        self.confirm_dialog =  self.wTree.get_object("confirm_dialog")
+        self.confirm_dialog = self.wTree.get_object("confirm_dialog")
         self.success_dialog = self.wTree.get_object("success_dialog")
 
-        if mode == "iso":
+        if mode_arg == "iso":
             self.mode = "normal"
             self.devicelist = self.wTree.get_object("device_combobox")
             self.label = self.wTree.get_object("to_label")
@@ -87,6 +97,11 @@ class MintStick:
             self.progressbar = self.wTree.get_object("progressbar")
             self.chooser = self.wTree.get_object("filechooserbutton")
 
+            # Making file chooser accessible for users with screen reader
+            label = self.wTree.get_object("label_write_image")
+            button = self.chooser.get_children()[0]
+            label.set_mnemonic_widget(button)
+
             # Devicelist model
             self.devicemodel = Gtk.ListStore(str, str)
 
@@ -108,24 +123,24 @@ class MintStick:
 
             # set callbacks
 
-            dict = {
-                    "on_cancel_button_clicked" : self.close,
-                    "on_emergency_button_clicked" : self.emergency_ok,
-                    "on_success_button_clicked" : self.success_ok,
-                    "on_confirm_cancel_button_clicked" : self.confirm_cancel}
-            self.wTree.connect_signals(dict)
+            callbacks = {
+                "on_cancel_button_clicked": self.close,
+                "on_emergency_button_clicked": self.emergency_ok,
+                "on_success_button_clicked": self.success_ok,
+                "on_confirm_cancel_button_clicked": self.confirm_cancel}
+            self.wTree.connect_signals(callbacks)
 
             self.devicelist.connect("changed", self.device_selected)
             self.go_button.connect("clicked", self.do_write)
             self.chooser.connect("file-set", self.file_selected)
 
-            if iso_path:
-                if os.path.exists(iso_path):
-                    self.chooser.set_filename(iso_path)
+            if iso_path_arg:
+                if os.path.exists(iso_path_arg):
+                    self.chooser.set_filename(iso_path_arg)
                     self.file_selected(self.chooser)
 
-        if mode == "format":
-            self.mode="format"
+        if mode_arg == "format":
+            self.mode = "format"
             self.devicelist = self.wTree.get_object("formatdevice_combobox")
             self.label = self.wTree.get_object("formatdevice_label")
             self.expander = self.wTree.get_object("formatdetail_expander")
@@ -141,12 +156,12 @@ class MintStick:
             self.progressbar = self.wTree.get_object("format_progressbar")
             self.filesystemlist = self.wTree.get_object("filesystem_combobox")
             # set callbacks
-            dict = {
-                    "on_cancel_button_clicked" : self.close,
-                    "on_emergency_button_clicked" : self.emergency_ok,
-                    "on_success_button_clicked" : self.success_ok,
-                    "on_confirm_cancel_button_clicked" : self.confirm_cancel}
-            self.wTree.connect_signals(dict)
+            callbacks = {
+                "on_cancel_button_clicked": self.close,
+                "on_emergency_button_clicked": self.emergency_ok,
+                "on_success_button_clicked": self.success_ok,
+                "on_confirm_cancel_button_clicked": self.confirm_cancel}
+            self.wTree.connect_signals(callbacks)
 
             self.go_button.connect("clicked", self.do_format)
             self.filesystemlist.connect("changed", self.filesystem_selected)
@@ -155,10 +170,10 @@ class MintStick:
             # Filesystemlist
             self.fsmodel = Gtk.ListStore(str, str, int, bool, bool)
             #                     id       label    max-length force-upper-case   force-alpha-numeric
-            self.fsmodel.append(["fat32", "FAT32",      11,        True,                True])
-            self.fsmodel.append(["exfat", "exFAT",      15,        False,               False])
-            self.fsmodel.append(["ntfs",  "NTFS",       32,        False,               False])
-            self.fsmodel.append(["ext4",  "EXT4",       16,        False,               False])
+            self.fsmodel.append(["fat32", "FAT32", 11, True, True])
+            self.fsmodel.append(["exfat", "exFAT", 15, False, False])
+            self.fsmodel.append(["ntfs", "NTFS", 32, False, False])
+            self.fsmodel.append(["ext4", "EXT4", 16, False, False])
             self.filesystemlist.set_model(self.fsmodel)
 
             # Renderer
@@ -177,27 +192,27 @@ class MintStick:
             self.filesystemlist.set_sensitive(True)
             # Default's to fat32
             self.filesystemlist.set_active(0)
-            if filesystem is not None:
-                iter = model.get_iter_first()
-                while iter is not None:
-                    value = model.get_value(iter, 0)
-                    if value == filesystem:
-                        self.filesystemlist.set_active_iter(iter)
-                    iter = model.iter_next(iter)
+            if filesystem_arg is not None:
+                itererator = self.fsmodel.get_iter_first()
+                while itererator is not None:
+                    value = self.fsmodel.get_value(itererator, 0)
+                    if value == filesystem_arg:
+                        self.filesystemlist.set_active_iter(itererator)
+                    itererator = self.fsmodel.iter_next(itererator)
 
             self.filesystem_selected(self.filesystemlist)
             self.get_devices()
 
-            if usb_path is not None:
-                iter = self.devicemodel.get_iter_first()
-                while iter is not None:
-                    value = self.devicemodel.get_value(iter, 0)
-                    if usb_path in value:
-                        self.devicelist.set_active_iter(iter)
-                    iter = self.devicemodel.iter_next(iter)
+            if usb_path_arg is not None:
+                itererator = self.devicemodel.get_iter_first()
+                while itererator is not None:
+                    value = self.devicemodel.get_value(itererator, 0)
+                    if usb_path_arg in value:
+                        self.devicelist.set_active_iter(itererator)
+                    itererator = self.devicemodel.iter_next(itererator)
 
         self.window.show_all()
-        if self.mode=="format":
+        if self.mode == "format":
             self.expander.hide()
         self.log = self.logview.get_buffer()
 
@@ -228,11 +243,11 @@ class MintStick:
                                 name = block.get_property('device')
                                 name = ''.join([i for i in name if not i.isdigit()])
 
-                            driveVendor = str(drive.get_property('vendor'))
-                            driveModel = str(drive.get_property('model'))
+                            drive_vendor = str(drive.get_property('vendor'))
+                            drive_model = str(drive.get_property('model'))
 
-                            if driveVendor.strip() != "":
-                                driveModel = "%s %s" % (driveVendor, driveModel)
+                            if drive_vendor.strip() != "":
+                                drive_model = "%s %s" % (drive_vendor, drive_model)
 
                             if size >= 1000000000000:
                                 size = "%.0fTB" % round(size / 1000000000000)
@@ -245,7 +260,7 @@ class MintStick:
                             else:
                                 size = "%.0fB" % round(size)
 
-                            item = "%s (%s) - %s" % (driveModel, name, size)
+                            item = "%s (%s) - %s" % (drive_model, name, size)
 
                             if item not in dct:
                                 dct.append(item)
@@ -254,18 +269,18 @@ class MintStick:
         self.devicelist.set_model(self.devicemodel)
 
     def device_selected(self, widget):
-        iter = self.devicelist.get_active_iter()
-        if iter is not None:
-            self.dev = self.devicemodel.get_value(iter, 0)
+        iterator = self.devicelist.get_active_iter()
+        if iterator is not None:
+            self.dev = self.devicemodel.get_value(iterator, 0)
             self.go_button.set_sensitive(True)
 
     def filesystem_selected(self, widget):
-        iter = self.filesystemlist.get_active_iter()
-        if iter is not None:
-            self.filesystem = self.fsmodel.get_value(iter, 0)
+        itererator = self.filesystemlist.get_active_iter()
+        if itererator is not None:
+            self.filesystem = self.fsmodel.get_value(itererator, 0)
             self.activate_devicelist()
 
-            self.label_entry.set_max_length(self.fsmodel.get_value(iter, 2))
+            self.label_entry.set_max_length(self.fsmodel.get_value(itererator, 2))
             self.on_label_entry_text_changed(self, self.label_entry)
 
     def file_selected(self, widget):
@@ -276,6 +291,7 @@ class MintStick:
 
         active_iter = self.filesystemlist.get_active_iter()
         value = self.fsmodel.get_value(active_iter, 0)
+        cursor_pos = self.label_entry.props.cursor_position
 
         if self.fsmodel.get_value(active_iter, 3):
             old_text = self.label_entry.get_text()
@@ -293,6 +309,7 @@ class MintStick:
 
         length = self.label_entry.get_buffer().get_length()
         self.label_entry.select_region(length, -1)
+        self.label_entry.set_position(cursor_pos)
 
         self.label_entry.handler_unblock(self.label_entry_changed_id)
 
@@ -326,21 +343,27 @@ class MintStick:
             self.pulse_progress()
             return True
         else:
-            GObject.idle_add(self.format_job_done, self.process.returncode)
+            GLib.idle_add(self.format_job_done, self.process.returncode)
             self.process = None
             return False
 
-    def raw_format(self, usb_path, fstype, label):
+    def raw_format(self, usb_path_arg, fstype, label):
         if os.geteuid() > 0:
-            launcher='pkexec'
-            self.process = Popen([launcher,'/usr/bin/python3', '-u', '/usr/lib/mintstick/raw_format.py','-d',usb_path,'-f',fstype, '-l', label, '-u', str(os.geteuid()), '-g', str(os.getgid())], shell=False, stdout=PIPE,  preexec_fn=os.setsid)
+            polkit_exec = 'pkexec'
+            self.process = Popen(
+                [polkit_exec, '/usr/bin/python3', '-u', '/usr/lib/mintstick/raw_format.py', '-d', usb_path_arg,
+                 '-f', fstype, '-l', label, '-u', str(os.geteuid()), '-g', str(os.getgid())],
+                shell=False, stdout=PIPE, preexec_fn=os.setsid)
         else:
-            self.process = Popen(['/usr/bin/python3', '-u', '/usr/lib/mintstick/raw_format.py','-d',usb_path,'-f',fstype, '-l', label, '-u', str(os.geteuid()), '-g', str(os.getgid())], shell=False, stdout=PIPE,  preexec_fn=os.setsid)
+            self.process = Popen(
+                ['/usr/bin/python3', '-u', '/usr/lib/mintstick/raw_format.py', '-d', usb_path_arg,
+                 '-f', fstype, '-l', label, '-u', str(os.geteuid()), '-g', str(os.getgid())],
+                shell=False, stdout=PIPE, preexec_fn=os.setsid)
 
         self.progressbar.show()
         self.pulse_progress()
 
-        GObject.timeout_add(500, self.check_format_job)
+        GLib.timeout_add(500, self.check_format_job)
 
     def format_job_done(self, rc):
         self.set_progress(1.0)
@@ -373,7 +396,7 @@ class MintStick:
         source = self.chooser.get_filename()
         target = self.dev
         self.logger(_('Image:') + ' ' + source)
-        self.logger(_('USB stick:')+ ' ' + self.dev)
+        self.logger(_('USB stick:') + ' ' + self.dev)
 
         if os.geteuid() > 0:
             self.raw_write(source, target)
@@ -389,8 +412,8 @@ class MintStick:
 
     def set_progress(self, size):
         self.progressbar.set_fraction(size)
-        str_progress = "%3.0f%%" % (float(size)*100)
-        int_progress = int(float(size)*100)
+        str_progress = "%3.0f%%" % (float(size) * 100)
+        int_progress = int(float(size) * 100)
         self.progressbar.set_text(str_progress)
         self.window.set_title("%s - %s" % (str_progress, _("USB Image Writer")))
         XApp.set_window_progress_pulse(self.window, False)
@@ -404,14 +427,15 @@ class MintStick:
     def update_progress(self, fd, condition):
         if Using_Unity:
             launcher.set_property("progress_visible", True)
-        if condition  is GLib.IO_IN:
+        if condition is GLib.IO_IN:
             line = fd.readline()
+            # noinspection PyBroadException
             try:
                 size = float(line.strip())
                 progress = round(size * 100)
                 if progress > self.write_progress:
                     self.write_progress = progress
-                    GObject.idle_add(self.set_progress, size)
+                    GLib.idle_add(self.set_progress, size)
                     if Using_Unity:
                         launcher.set_property("progress", size)
             except:
@@ -426,24 +450,30 @@ class MintStick:
         if self.process.returncode is None:
             return True
         else:
-            GObject.idle_add(self.write_job_done, self.process.returncode)
+            GLib.idle_add(self.write_job_done, self.process.returncode)
             self.process = None
             return False
 
     def raw_write(self, source, target):
         self.progressbar.set_sensitive(True)
-        self.progressbar.set_text(_('Writing %(VAR_FILE)s to %(VAR_DEV)s') % {'VAR_FILE': source.split('/')[-1], 'VAR_DEV': self.dev})
-        self.logger(_('Starting copy from %(VAR_SOURCE)s to %(VAR_TARGET)s') % {'VAR_SOURCE':source, 'VAR_TARGET':target})
+        self.progressbar.set_text(
+            _('Writing %(VAR_FILE)s to %(VAR_DEV)s') % {'VAR_FILE': source.split('/')[-1], 'VAR_DEV': self.dev})
+        self.logger(
+            _('Starting copy from %(VAR_SOURCE)s to %(VAR_TARGET)s') % {'VAR_SOURCE': source, 'VAR_TARGET': target})
 
         if os.geteuid() > 0:
-            launcher='pkexec'
-            self.process = Popen([launcher,'/usr/bin/python3', '-u', '/usr/lib/mintstick/raw_write.py','-s',source,'-t',target], shell=False, stdout=PIPE, preexec_fn=os.setsid)
+            polkit_exec = 'pkexec'
+            self.process = Popen(
+                [polkit_exec, '/usr/bin/python3', '-u', '/usr/lib/mintstick/raw_write.py', '-s', source, '-t', target],
+                shell=False, stdout=PIPE, preexec_fn=os.setsid)
         else:
-            self.process = Popen(['/usr/bin/python3', '-u', '/usr/lib/mintstick/raw_write.py','-s',source,'-t',target], shell=False, stdout=PIPE, preexec_fn=os.setsid)
+            self.process = Popen(
+                ['/usr/bin/python3', '-u', '/usr/lib/mintstick/raw_write.py', '-s', source, '-t', target],
+                shell=False, stdout=PIPE, preexec_fn=os.setsid)
 
         self.write_progress = 0
-        self.source_id = GLib.io_add_watch(self.process.stdout, GLib.IO_IN|GLib.IO_HUP, self.update_progress)
-        GObject.timeout_add(500, self.check_write_job)
+        self.source_id = GLib.io_add_watch(self.process.stdout, GLib.IO_IN | GLib.IO_HUP, self.update_progress)
+        GLib.timeout_add(500, self.check_write_job)
 
     def write_job_done(self, rc):
         if rc == 0:
@@ -467,7 +497,7 @@ class MintStick:
         self.emergency(message)
         return False
 
-    def success(self,message):
+    def success(self, message):
         label = self.wTree.get_object("label5")
         label.set_text(message)
         if self.mode == "normal":
@@ -481,7 +511,7 @@ class MintStick:
             self.final_unsensitive()
         label = self.wTree.get_object("label6")
         label.set_text(message)
-        #self.expander.set_expanded(True)
+        # self.expander.set_expanded(True)
         mark = self.log.create_mark("end", self.log.get_end_iter(), False)
         self.logview.scroll_to_mark(mark, 0.05, True, 0.0, 1.0)
         resp = self.emergency_dialog.run()
@@ -513,26 +543,29 @@ class MintStick:
         print(self.log.get_text(start, end, False))
 
     def logger(self, text):
-        self.log.insert_at_cursor(text+"\n")
+        self.log.insert_at_cursor(text + "\n")
 
     def activate_devicelist(self):
         self.devicelist.set_sensitive(True)
         self.expander.set_sensitive(True)
         self.label.set_sensitive(True)
 
-    def confirm_cancel(self,widget):
+    def confirm_cancel(self, widget):
         self.confirm_dialog.hide()
-        if self.mode == "normal": self.set_iso_sensitive()
-        if self.mode == "format": self.set_format_sensitive()
+        if self.mode == "normal":
+            self.set_iso_sensitive()
+        if self.mode == "format":
+            self.set_format_sensitive()
 
-    def emergency_ok(self,widget):
+    def emergency_ok(self, widget):
         self.emergency_dialog.hide()
-        if self.mode == "normal": self.set_iso_sensitive()
+        if self.mode == "normal":
+            self.set_iso_sensitive()
         if self.mode == "format":
             self.set_format_sensitive()
             self.go_button.set_sensitive(False)
 
-    def success_ok(self,widget):
+    def success_ok(self, widget):
         self.success_dialog.hide()
         if self.mode == "normal":
             self.set_iso_sensitive()
@@ -551,22 +584,25 @@ class MintStick:
         self.devicelist.set_sensitive(True)
         self.go_button.set_sensitive(True)
 
+
 if __name__ == "__main__":
 
-    usb_path=None
-    iso_path=None
-    filesystem=None
-    mode=None
+    usb_path = None
+    iso_path = None
+    filesystem = None
+    mode = None
+
 
     def usage():
         print("Usage: mintstick [--debug] -m [format|iso]              : mode (format usb stick or burn iso image)")
         print("       mintstick [--debug] -m iso [-i|--iso] iso_path")
         print("       mintstick [--debug] -m format [-u|--usb] usb_device ")
         print("                           [-f|--filesystem] filesystem")
-        exit (0)
+        exit(0)
+
 
     try:
-        opts, args = getopt.getopt(sys.argv[1:], "hm:i:u:f:", ["debug", "help", "mode=", "iso=","usb=","filesystem="])
+        opts, args = getopt.getopt(sys.argv[1:], "hm:i:u:f:", ["debug", "help", "mode=", "iso=", "usb=", "filesystem="])
     except getopt.error as msg:
         print(msg)
         print("for help use --help")
@@ -579,14 +615,15 @@ if __name__ == "__main__":
         elif o in ("-i", "--iso"):
             iso_path = a
         elif o in ("-u", "--usb"):
-            # hack. KDE Solid application gives partition name not device name. Need to remove extra digit from the string.
+            # hack. KDE Solid application gives partition name not device name.
+            # Need to remove extra digit from the string.
             # ie. /dev/sdj1 -> /dev/sdj
             usb_path = ''.join([i for i in a if not i.isdigit()])
         elif o in ("-f", "--filesystem"):
             filesystem = a
         elif o in ("-m", "--mode"):
-            mode=a
-        elif o in ("--debug"):
+            mode = a
+        elif o in "--debug":
             debug = True
 
     argc = len(sys.argv)
@@ -601,7 +638,7 @@ if __name__ == "__main__":
 
     MintStick(iso_path, usb_path, filesystem, mode, debug)
 
-    #start the main loop
-    #mainloop = GObject.MainLoop()
-    #mainloop.run()
+    # start the main loop
+    # mainloop = GObject.MainLoop()
+    # mainloop.run()
     Gtk.main()
